@@ -1,8 +1,8 @@
 /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
+   334 Harvard street, Cambridge, MA 02139 USA
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,8 +21,11 @@
 
 */
 
-#include "igraph.h"
-#include "memory.h"
+#include "igraph_community.h"
+#include "igraph_memory.h"
+#include "igraph_interface.h"
+#include "igraph_iterators.h"
+#include "config.h"
 
 /**
  * \function igraph_coreness 
