@@ -1,8 +1,8 @@
 /* -*- mode: C -*-  */
 /* 
    IGraph library.
-   Copyright (C) 2006  Gabor Csardi <csardi@rmki.kfki.hu>
-   MTA RMKI, Konkoly-Thege Miklos st. 29-33, Budapest 1121, Hungary
+   Copyright (C) 2006-2012  Gabor Csardi <csardi.gabor@gmail.com>
+   334 Harvard street, Cambridge, MA 02139 USA
    
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,9 +21,14 @@
 
 */
 
-#include "igraph.h"
-#include "error.h"
-#include "memory.h"
+#include "igraph_operators.h"
+#include "igraph_error.h"
+#include "igraph_memory.h"
+#include "igraph_interrupt_internal.h"
+#include "igraph_interface.h"
+#include "igraph_constructors.h"
+#include "igraph_adjlist.h"
+#include "igraph_attributes.h"
 #include "config.h"
 
 /**
@@ -56,6 +61,8 @@
  * union.
  * 
  * Time complexity: O(|V1|+|V2|+|E1|+|E2|).
+ * 
+ * \example examples/simple/igraph_disjoint_union.c
  */
 
 int igraph_disjoint_union(igraph_t *res, const igraph_t *left, 
@@ -101,7 +108,7 @@ int igraph_disjoint_union(igraph_t *res, const igraph_t *left,
  * \brief The disjint union of many graphs.
  *
  * </para><para>
- * First the vertices in the graphs will be relabed with new vertex
+ * First the vertices in the graphs will be relabeled with new vertex
  * ids to have pairwise disjoint vertex id sets and then the union of
  * the graphs is formed.
  * The number of vertices and edges in the result is the total number
@@ -193,6 +200,8 @@ int igraph_disjoint_union_many(igraph_t *res,
  * Time complexity: O(|V|+|E|), |V| is the number of nodes, |E|
  * is the number of edges in the smaller graph of the two. (The one 
  * containing less vertices is considered smaller.)
+ * 
+ * \example examples/simple/igraph_intersection.c
  */
 
 int igraph_intersection(igraph_t *res,
@@ -447,6 +456,8 @@ int igraph_intersection_many(igraph_t *res,
  * 
  * Time complexity: O(|V|+|E|), |V| is the number of
  * vertices, |E| the number of edges in the result graph.
+ * 
+ * \example examples/simple/igraph_union.c
  */
 
 int igraph_union(igraph_t *res, 
@@ -533,7 +544,7 @@ void igraph_i_union_many_free(igraph_vector_ptr_t *v) {
  * 
  * </para><para> 
  * The result graph will contain as many vertices as the largest graph
- * among the agruments does, and an edge will be included in it if it
+ * among the arguments does, and an edge will be included in it if it
  * is part of at least one operand graph.
  * 
  * </para><para>
@@ -551,6 +562,8 @@ void igraph_i_union_many_free(igraph_vector_ptr_t *v) {
  * 
  * Time complexity: O(|V|+|E|), |V| is the number of vertices
  * in largest graph and |E| is the number of edges in the result graph.
+ * 
+ * \example examples/simple/igraph_union.c
  */
 
 int igraph_union_many(igraph_t *res, const igraph_vector_ptr_t *graphs) {
@@ -691,17 +704,26 @@ int igraph_union_many(igraph_t *res, const igraph_vector_ptr_t *graphs) {
  * Time complexity: O(|V|+|E|), |V| is the number vertices in
  * the smaller graph, |E| is the
  * number of edges in the result graph.
+ * 
+ * \example examples/simple/igraph_difference.c
  */
 
 int igraph_difference(igraph_t *res, 
 		      const igraph_t *orig, const igraph_t *sub) {
+
+  /* Quite nasty, but we will use that an edge adjacency list
+     contains the vertices according to the order of the 
+     vertex ids at the "other" end of the edge. */
+
   long int no_of_nodes_orig=igraph_vcount(orig);
   long int no_of_nodes_sub =igraph_vcount(sub);
   long int no_of_nodes=no_of_nodes_orig;
   long int smaller_nodes;
   igraph_bool_t directed=igraph_is_directed(orig);
   igraph_vector_t edges;
-  igraph_vector_t nei1, nei2;
+  igraph_vector_t edge_ids;
+  igraph_vector_t *nei1, *nei2;
+  igraph_inclist_t inc_orig, inc_sub;
   long int i;
   igraph_integer_t v1, v2;
 
@@ -710,63 +732,93 @@ int igraph_difference(igraph_t *res,
 		 IGRAPH_EINVAL);
   }
   
+  IGRAPH_VECTOR_INIT_FINALLY(&edge_ids, 0);
   IGRAPH_VECTOR_INIT_FINALLY(&edges, 0);
-  IGRAPH_VECTOR_INIT_FINALLY(&nei1, 0);
-  IGRAPH_VECTOR_INIT_FINALLY(&nei2, 0);
+  IGRAPH_CHECK(igraph_inclist_init(orig, &inc_orig, IGRAPH_OUT));
+  IGRAPH_FINALLY(igraph_inclist_destroy, &inc_orig);
+  IGRAPH_CHECK(igraph_inclist_init(sub, &inc_sub, IGRAPH_OUT));
+  IGRAPH_FINALLY(igraph_inclist_destroy, &inc_sub);
   
   smaller_nodes=no_of_nodes_orig > no_of_nodes_sub ?
     no_of_nodes_sub : no_of_nodes_orig;
   
   for (i=0; i<smaller_nodes; i++) {
+    long int n1, n2, e1, e2;
     IGRAPH_ALLOW_INTERRUPTION();
-    IGRAPH_CHECK(igraph_neighbors(orig, &nei1, i, IGRAPH_OUT));
-    IGRAPH_CHECK(igraph_neighbors(sub, &nei2, i, IGRAPH_OUT));
-    if (!directed) {
-      igraph_vector_filter_smaller(&nei1, i);
-      igraph_vector_filter_smaller(&nei2, i);
-    }
-    while (!igraph_vector_empty(&nei1) && !igraph_vector_empty(&nei2)) {
-      v1=igraph_vector_tail(&nei1);
-      v2=igraph_vector_tail(&nei2);
+    nei1=igraph_inclist_get(&inc_orig, i);
+    nei2=igraph_inclist_get(&inc_sub, i);
+    n1=igraph_vector_size(nei1)-1;
+    n2=igraph_vector_size(nei2)-1;
+    while (n1>=0 && n2>=0) {
+      e1=VECTOR(*nei1)[n1];
+      e2=VECTOR(*nei2)[n2];
+      v1=IGRAPH_OTHER(orig, e1, i);
+      v2=IGRAPH_OTHER(sub, e2, i);
       
-      if (v1>v2) {
+      if (!directed && v1<i) { 
+	n1--;
+      } else if (!directed && v2<i) {
+	n2--;
+      } else if (v1>v2) {
+	IGRAPH_CHECK(igraph_vector_push_back(&edge_ids, e1));
 	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
 	IGRAPH_CHECK(igraph_vector_push_back(&edges, v1));
-	igraph_vector_pop_back(&nei1);
+	n1--;	
       } else if (v2>v1) {
-	igraph_vector_pop_back(&nei2);
+	n2--;
       } else {
-	igraph_vector_pop_back(&nei1);
-	igraph_vector_pop_back(&nei2);
+	n1--;
+	n2--;
       }
     }
-
+    
     /* Copy remaining edges */
-    while (!igraph_vector_empty(&nei1)) {
-      IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
-      IGRAPH_CHECK(igraph_vector_push_back(&edges, 
-					   igraph_vector_pop_back(&nei1)));
+    while (n1>=0) {
+      e1=VECTOR(*nei1)[n1];
+      v1=IGRAPH_OTHER(orig, e1, i);
+      if (directed || v1 >= i) { 
+	IGRAPH_CHECK(igraph_vector_push_back(&edge_ids, e1));
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, v1));
+      }
+      n1--;
     }
   }
 
   /* copy remaining edges, use the previous value of 'i' */
   for (; i<no_of_nodes_orig; i++) {
-    IGRAPH_CHECK(igraph_neighbors(orig, &nei1, i, IGRAPH_OUT));
-    if (!directed) {
-      igraph_vector_filter_smaller(&nei1, i);
-    }
-    while (!igraph_vector_empty(&nei1)) {
-      IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
-      IGRAPH_CHECK(igraph_vector_push_back(&edges, 
-					   igraph_vector_pop_back(&nei1)));
+    long int n1, e1;
+    nei1=igraph_inclist_get(&inc_orig, i);
+    n1=igraph_vector_size(nei1)-1;
+    while (n1>=0) {
+      e1=VECTOR(*nei1)[n1];
+      v1=IGRAPH_OTHER(orig, e1, i);
+      if (directed || v1 >= i) { 
+	IGRAPH_CHECK(igraph_vector_push_back(&edge_ids, e1));
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, i));
+	IGRAPH_CHECK(igraph_vector_push_back(&edges, v1));
+      }
+      n1--;
     }
   }
 
+  igraph_inclist_destroy(&inc_sub);
+  igraph_inclist_destroy(&inc_orig);
+  IGRAPH_FINALLY_CLEAN(2);
   IGRAPH_CHECK(igraph_create(res, &edges, no_of_nodes, directed));
-  igraph_vector_destroy(&edges);
-  igraph_vector_destroy(&nei1);
-  igraph_vector_destroy(&nei2);
-  IGRAPH_FINALLY_CLEAN(3);
+  igraph_vector_destroy(&edges);  
+  IGRAPH_FINALLY_CLEAN(1);
+
+  /* Attributes */
+  if (orig->attr) {
+    IGRAPH_I_ATTRIBUTE_DESTROY(res);
+    IGRAPH_I_ATTRIBUTE_COPY(res, orig, /*graph=*/1, /*vertex=*/1, /*edge=*/0);
+    IGRAPH_CHECK(igraph_i_attribute_permute_edges(orig, res, &edge_ids));
+  }
+  
+  igraph_vector_destroy(&edge_ids);
+  IGRAPH_FINALLY_CLEAN(1);
+  
   return 0;
 }
 
@@ -787,6 +839,8 @@ int igraph_difference(igraph_t *res,
  * Time complexity: O(|V|+|E1|+|E2|), |V| is the number of
  * vertices in the graph, |E1| is the number of edges in the original
  * and |E2| in the complementer graph.
+ * 
+ * \example examples/simple/igraph_complementer.c
  */
 
 int igraph_complementer(igraph_t *res, const igraph_t *graph, 
@@ -834,9 +888,11 @@ int igraph_complementer(igraph_t *res, const igraph_t *graph,
   }
   
   IGRAPH_CHECK(igraph_create(res, &edges, no_of_nodes, 
-			     igraph_is_directed(graph)));
+			     igraph_is_directed(graph)));  
   igraph_vector_destroy(&edges);
   igraph_vector_destroy(&neis);
+  IGRAPH_I_ATTRIBUTE_DESTROY(res);
+  IGRAPH_I_ATTRIBUTE_COPY(res, graph, /*graph=*/1, /*vertex=*/1, /*edge=*/0);
   IGRAPH_FINALLY_CLEAN(2);
   return 0;
 }
@@ -860,13 +916,15 @@ int igraph_complementer(igraph_t *res, const igraph_t *graph,
  * 
  * \param res Pointer to an uninitialized graph object, the result
  *        will be stored here.
- * \param g1 The firs operarand, a graph object.
+ * \param g1 The firs operand, a graph object.
  * \param g2 The second operand, another graph object.
  * \return Error code.
  * 
  * Time complexity: O(|V|*d1*d2), |V| is the number of vertices in the
  * first graph, d1 and d2 the average degree in the first and second
  * graphs. 
+ * 
+ * \example examples/simple/igraph_compose.c
  */
 
 int igraph_compose(igraph_t *res, const igraph_t *g1, const igraph_t *g2) {
